const fs = require('fs').promises;

import { version, Sensor, Datum, SensorTimeSeries, SensorDatum, SensorTemperature, SensorFan, SensorDoor,getSensorFromObjectAuto, } from '.';

let data;
beforeAll(async () => {
  data = await fs.readFile('./resources/sensors_data.json', {
    encoding: 'utf8',
  });
  data = JSON.parse(data);
});

describe('Sensor model tests', () => {
  describe('Dummy tests', () => {
    test('data is loaded with 3 elements', () => {
      expect(data.length).toBe(3);
    });
    test('version number from the model', () => {
      expect(version()).toBe('1.0.0');
    });
  });

  let objectTemp = data[0];
  let objectDoor = data[1];
  let objectFan = data[2];

  describe('information Sensor', () => {
    test('id', () => {
      expect(Sensor.toObj(objectDoor).id).toBe('10245');
    });
    test('name', () => {
      expect(Sensor.toObj(objectDoor).name).toBe('Porte du Garage');
    });
    test('data', () => {
      expect(Sensor.toObj(objectDoor).data).toStrictEqual({ value: 0 });
    });
  });

  describe('Sensor tests', () => {
    test('Object to Sensor instance conversion', () => {
      data.forEach((item) =>
        expect(Sensor.toObj(item)).toBeInstanceOf(Sensor)
      );
    });
    test('Object to SensorTimeSeries instance conversion', () => {
      expect(SensorTimeSeries.toObj(objectTemp)).toBeInstanceOf(
        SensorTimeSeries);
      expect(SensorTimeSeries.toObj(objectFan)).toBeInstanceOf(
        SensorTimeSeries);
    });
    test('Object to SensorDatum instance conversion', () => {
      expect(SensorDatum.toObj(objectDoor)).toBeInstanceOf(SensorDatum);
    });
  });

  describe('data error', () => {
    test('SensorDatum', () => {
      expect(() => new SensorDatum('l', 'l', 1)).toThrow();
    });
    test('TimeSeries', () => {
      expect(() => new SensorTimeSeries('l', 'l', 1)).toThrow();
    });
    test('set data SensorDatum', () => {
      expect(() => (SensorDatum.toObj(objectDoor).data = {})).toThrow();
    });
    test('set data TimeSeries', () => {
      expect(
        () => (SensorTimeSeries.toObj(objectFan).data = {})
      ).toThrow();
    });
  });

  describe('classe Sensor  error', () => {
    test('id and name must be strings', () => {
      expect(() => new Sensor(1, 'l')).toThrow();
      expect(() => new Sensor('l', 1)).toThrow();
    });
    test('data must be an object', () => {
      expect(() => new Sensor('l', 'l', 1)).toThrow();
    });
  });

  describe('Automatic conversion from an object', () => {
    test('Object to SensorDoor', () => {
      expect(getSensorFromObjectAuto(objectDoor)).toBeInstanceOf(SensorDoor);
    });
    test('Object to SensorTemperature', () => {
      expect(getSensorFromObjectAuto(objectTemp)).toBeInstanceOf(
        SensorTemperature
      );
    });
    test('Object to SensorFan', () => {
      expect(getSensorFromObjectAuto(objectFan)).toBeInstanceOf(SensorFan);
    });
    test('object inconnu', () => {
      expect(() => getSensorFromObjectAuto({ type: 'L' })).toThrow();
    });
  });

  describe('Object to Datum instance', () => {
    test('Check instance', () => {
      expect(Datum.toObj({ value: 1 })).toBeInstanceOf(Datum);
    });
  });


  describe('access to data from sensors', () => {
    test('Est ce que la porte est ouverte ?', () => {
      expect(getSensorFromObjectAuto(objectDoor).isDoorOpen).toBeFalsy();
    });
  });

  describe('ToString tests', () => {
    test('Sensor toString', () => {
      expect(Sensor.toObj(objectDoor).toString()).toBe(
        'Porte du Garage@10245'
      );
    });
    let SensorFan = SensorTimeSeries.toObj(objectFan);
    test('SensorTimeSeries toString', () => {
      expect(SensorFan.toString()).toBe(
        `Ventilateur Ordinateur Bureau@2222 {\n\t"2021-01-19T10:00:00.000Z": 1073,\n\t"2021-01-19T10:05:00.000Z": 1800,\n\t"2021-01-19T10:10:00.000Z": 2299,\n\t"2021-01-19T10:15:00.000Z": 2176,\n\t"2021-01-19T10:20:00.000Z": 1899,\n\t"2021-01-19T10:25:00.000Z": 1400,\n}`
      );
    });
    test('SensorFan toString', () => {
      expect(SensorFan.getFromSensor(SensorFan).toString()).toBe(
        `Ventilateur Ordinateur Bureau@2222 {\n\t"2021-01-19T10:00:00.000Z": 1073 rpm,\n\t"2021-01-19T10:05:00.000Z": 1800 rpm,\n\t"2021-01-19T10:10:00.000Z": 2299 rpm,\n\t"2021-01-19T10:15:00.000Z": 2176 rpm,\n\t"2021-01-19T10:20:00.000Z": 1899 rpm,\n\t"2021-01-19T10:25:00.000Z": 1400 rpm,\n}`
      );
    });
    test('Temperature toString', () => {
      expect(SensorTemperature.toObj(objectTemp).toString()).toBe(
        `Température Bureau@1234 {\n\t"2021-01-19T08:00:00.000Z": 23 °C,\n\t"2021-01-19T09:00:00.000Z": 23 °C,\n\t"2021-01-19T10:00:00.000Z": 22 °C,\n\t"2021-01-19T11:00:00.000Z": 21 °C,\n\t"2021-01-19T12:00:00.000Z": 23 °C,\n\t"2021-01-19T13:00:00.000Z": 23 °C,\n\t"2021-01-19T14:00:00.000Z": 23 °C,\n\t"2021-01-19T15:00:00.000Z": 25 °C,\n\t"2021-01-19T16:00:00.000Z": 25 °C,\n}`
      );
    });
  });


});
